/***************************************************************************
 *   Copyright (C) 2008 by Matthieu Gallien <matthieu_gallien@yahoo.fr>    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#ifndef ABSTRACTTASKSAPPLET_H
#define ABSTRACTTASKSAPPLET_H

#include "plasma_tasks_lib_export.h"

#include <plasma/applet.h>
#include <plasma/dataengine.h>

#include <QHash>
#include <QString>

class TasksListView;
class TasksModel;
class QGraphicsLinearLayout;
class AbstractTaskWidgetFactory;

class PLASMA_TASKS_LIB_EXPORT AbstractTasksApplet : public Plasma::Applet
{
    Q_OBJECT
public:
    ~AbstractTasksApplet();

    void init();

    qreal itemLeftMargin() const;
    qreal itemRightMargin() const;
    qreal itemTopMargin() const;
    qreal itemBottomMargin() const;

    void constraintsEvent(Plasma::Constraints constraints);

protected slots:

    void taskOrStartupAdded(const QString& source);

    void taskOrStartupRemoved(const QString& source);

    void dataUpdated(const QString &sourceName, const Plasma::DataEngine::Data &data);

protected:

    AbstractTasksApplet(QObject* parentObject, const QVariantList& args);

    virtual AbstractTaskWidgetFactory* createTaskWidgetFactory() = 0;

private:
    TasksModel *m_tasksModel;
    TasksListView *m_list;

    qreal m_leftMargin;
    qreal m_topMargin;
    qreal m_rightMargin;
    qreal m_bottomMargin;
};

#endif
