/***************************************************************************
 *   Copyright (C) 2008 by Matthieu Gallien <matthieu_gallien@yahoo.fr>    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#include "classical_task_widget.h"
#include "tasks_model.h"
#include "abstract_tasks_applet.h"

#include <plasma/theme.h>
#include <taskmanager/task.h>
#include <KDebug>
#include <KGlobalSettings>

#include <QPainter>
#include <QIcon>
#include <QStyleOptionGraphicsItem>
#include <QTextLayout>
#include <QApplication>

ClassicalTaskWidget::ClassicalTaskWidget(AbstractTasksApplet *applet,
                                         const QPersistentModelIndex &p_index)
    : AbstractTaskWidget(applet, p_index)
{
}

void ClassicalTaskWidget::paint(QPainter *painter, const QStyleOptionGraphicsItem *option,
                                QWidget *widget)
{
    painter->setRenderHint(QPainter::Antialiasing);
    painter->setRenderHint(QPainter::TextAntialiasing);

    QRectF contentRect = contentsRect();

    drawBackground(contentRect, painter, option, widget);

    Q_UNUSED(option)

    QRectF bounds = boundingRect().adjusted(applet()->itemLeftMargin(), applet()->itemTopMargin(), -applet()->itemRightMargin(), -applet()->itemBottomMargin());
    drawTaskIcon(iconRect(bounds).toRect(), painter, option, widget);

    painter->setPen(QPen(Plasma::Theme::defaultTheme()->color(Plasma::Theme::TextColor), 1.0));

    QRect rect = textRect(bounds).toRect();
    rect.adjust(2, 2, -2, -2); // Create a text margin

    QTextLayout layout;
    layout.setFont(KGlobalSettings::taskbarFont());
    layout.setTextOption(textOption());

    layoutText(layout, index().data(TaskRoles::TaskName).toString(), rect.size());
    drawTextLayout(painter, layout, rect);
}

QRectF ClassicalTaskWidget::iconRect(const QRectF &b) const
{
    QRectF bounds(b);
    const int right = bounds.right();
    //leave enough space for the text. useful in vertical panel
    bounds.setWidth(qMax(bounds.width() / 3, qMin(minimumSize().height(), bounds.width())));

    //restore right position if the layout is RTL
    if (QApplication::layoutDirection() == Qt::RightToLeft) {
        bounds.moveRight(right);
    }

    QSize iconSize = index().data(TaskRoles::TaskIcon).value<QIcon>().actualSize(bounds.size().toSize());

    return QStyle::alignedRect(QApplication::layoutDirection(), Qt::AlignLeft | Qt::AlignVCenter,
                               iconSize, bounds.toRect());
}

QRectF ClassicalTaskWidget::textRect(const QRectF &bounds) const
{
    QSize size(bounds.size().toSize());
    size.rwidth() -= int(iconRect(bounds).width()) + qMax(0, IconTextSpacing - 2);

    return QStyle::alignedRect(QApplication::layoutDirection(), Qt::AlignRight | Qt::AlignVCenter,
                                     size, bounds.toRect());
}
