/***************************************************************************
 *   Copyright (C) 2008 by Matthieu Gallien <matthieu_gallien@yahoo.fr>    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#include "compact_task_widget.h"
#include "tasks_model.h"

#include <taskmanager/task.h>
#include <KDebug>

#include <QPainter>
#include <QIcon>
#include <QStyleOptionGraphicsItem>

CompactTaskWidget::CompactTaskWidget(AbstractTasksApplet *applet,
                                     const QPersistentModelIndex &p_index)
    : AbstractTaskWidget(applet, p_index)
{
}

void CompactTaskWidget::paint(QPainter *painter, const QStyleOptionGraphicsItem *option,
                             QWidget *widget)
{
    painter->setRenderHint(QPainter::Antialiasing);
    painter->setRenderHint(QPainter::TextAntialiasing);

    QRectF contentRect = contentsRect();

    drawBackground(contentRect, painter, option, widget);

    const QRectF iconRect(contentRect);
    drawTaskIcon(iconRect, painter, option, widget);

    const QSizeF desktopNumberMargin(2.0, 2.0);
    const QSizeF &desktopNumberSize(getDesktopNumberSize(option));
    const QRectF desktopNumberRect(contentRect.x() + (desktopNumberMargin.width() / 2), contentRect.y() + (desktopNumberMargin.height() / 2),
                                   desktopNumberSize.height() * 3 / 4, desktopNumberSize.height() * 3 / 4);
    drawDesktopNumber(desktopNumberRect, painter, option, widget);

    const QSizeF minimizedIndicatorMargin(2.0, 2.0);
    const QSizeF minimizedIndicatorSize(8.0, 8.0);
    const QRectF minimizedIndicatorRect(contentRect.x() + minimizedIndicatorMargin.width(),
                                        contentRect.y() - minimizedIndicatorMargin.width() + contentRect.height() - minimizedIndicatorSize.height(),
                                        minimizedIndicatorSize.width(), minimizedIndicatorSize.height());
    drawMinimizedFlag(minimizedIndicatorRect, painter, option, widget);
}

QSizeF CompactTaskWidget::sizeHint(Qt::SizeHint which, const QSizeF & constraint) const
{
    kDebug() << "new widget size:" << QSizeF(mainSize(), mainSize());
    return QSizeF(mainSize(), mainSize());
}
