/***************************************************************************
 *   Copyright (C) 2008 by Matthieu Gallien <matthieu_gallien@yahoo.fr>    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#ifndef TASKSLISTVIEWAPPLET_H
#define TASKSLISTVIEWAPPLET_H

#include "plasma_tasks_lib_export.h"

#include <plasma/plasma.h>

#include <QGraphicsWidget>

class QAbstractItemModel;
class QModelIndex;
class QGraphicsItem;
class QGraphicsLinearLayout;
class AbstractTaskWidgetFactory;
class AbstractTasksApplet;

class PLASMA_TASKS_LIB_EXPORT TasksListView : public QObject
{

    Q_OBJECT

public:

    explicit TasksListView(AbstractTasksApplet *parent);

    QGraphicsLinearLayout* getLayout() const;

    void init(AbstractTaskWidgetFactory *factory, QAbstractItemModel *p_model);

    void changeWidgetFactory(AbstractTaskWidgetFactory *factory);

    void changeModel(QAbstractItemModel *p_model);

    QAbstractItemModel* model() const;
    
public slots:
    
    void sizeChanged(double);

protected slots:

    void dataChanged(const QModelIndex &topLeft, const QModelIndex &bottomRight);
    void layoutChanged();
    void modelReset();
    void rowsInserted(const QModelIndex &parent, int start, int end);
    void rowsRemoved(const QModelIndex &parent, int start, int end);

protected:

    void setModel(QAbstractItemModel *p_model);

private:

    QAbstractItemModel *m_model;

    QGraphicsLinearLayout *m_layout;

    AbstractTasksApplet *m_parent;

    QMap<int, QGraphicsWidget*> m_widget;

    AbstractTaskWidgetFactory *m_widgetFactory;
    
    double m_mainSize;

};

#endif
