/***************************************************************************
 *   Copyright (C) 2008 by Matthieu Gallien <matthieu_gallien@yahoo.fr>    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#ifndef TASKSMODELAPPLET_H
#define TASKSMODELAPPLET_H

#include "plasma_tasks_lib_export.h"

#include <QAbstractListModel>
#include <QHash>
#include <QString>
#include <QVariant>
#include <QMap>

namespace TaskRoles
{

    const int TaskName = 1;

    const int TaskIcon = 2;

    const int DesktopNumber = 3;

    const int IsMinimized = 4;

    const int IsActive = 5;

    const int TaskDemandsAttention = 6;

    const int WId = 7;

    const int IsTask = 8;

    const int IsStartup = 9;

}

class PLASMA_TASKS_LIB_EXPORT TasksModel : public QAbstractListModel
{

    Q_OBJECT

public:

    TasksModel(QObject* parent = 0);

    Qt::ItemFlags flags(const QModelIndex& index) const;

    QVariant data(const QModelIndex& index, int role = Qt::DisplayRole) const;

    int rowCount(const QModelIndex& index = QModelIndex()) const;

    QModelIndex indexFromId(const QString &id) const;

    bool existsTaskOrStartup(const QString &id) const;

    bool isStartup(const QString &id) const;

    bool isTask(const QString &id) const;

public slots:

    void addTaskOrStartup(const QString&, const QHash<QString, QVariant> &taskData);

    bool removeTaskOrStartup(const QString&);

    void updateTaskOrStartup(const QString&, const QHash<QString, QVariant> &taskData);

protected:

    int positionById(const QString &id) const;

    QString idByPosition(int) const;

    QHash<QString, QVariant> taskOrStartupById(const QString &id) const;

private:

    QMap<QString, QHash<QString, QVariant> > m_tasksOrStartups;

    QMap<QString, int> m_positionById;

    QMap<int, QString> m_idByPosition;

};

#endif
