/***************************************************************************
 *   Copyright (C) 2008 by Matthieu Gallien <matthieu_gallien@yahoo.fr>    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#include "tasksmodeltest.h"
#include "tasks_model.h"
#include <QModelIndex>
#include <QVariant>

#include <QtTest>
#include <QtCore>
#include <QtGui>

void TasksModelUnitTests::testTaskAdd()
{
    TasksModel myModel;

    QSignalSpy rowsAboutToBeInsertedSpy(&myModel, SIGNAL(rowsAboutToBeInserted(const QModelIndex&, int, int)));
    QSignalSpy rowsInsertedSpy(&myModel, SIGNAL(rowsInserted(const QModelIndex&, int, int)));
    QSignalSpy rowsAboutToBeRemovedSpy(&myModel, SIGNAL(rowsAboutToBeRemoved(const QModelIndex&, int, int)));
    QSignalSpy rowsRemovedSpy(&myModel, SIGNAL(rowsRemoved(const QModelIndex&, int, int)));
    QSignalSpy dataChangedSpy(&myModel, SIGNAL(dataChanged(const QModelIndex&, const QModelIndex&)));
    QSignalSpy layoutChangedSpy(&myModel, SIGNAL(layoutChanged()));
    QSignalSpy modelResetSpy(&myModel, SIGNAL(modelReset()));

    QVERIFY(rowsAboutToBeInsertedSpy.isValid());
    QVERIFY(rowsInsertedSpy.isValid());
    QVERIFY(rowsAboutToBeRemovedSpy.isValid());
    QVERIFY(rowsRemovedSpy.isValid());
    QVERIFY(dataChangedSpy.isValid());
    QVERIFY(layoutChangedSpy.isValid());
    QVERIFY(modelResetSpy.isValid());

    myModel.addTaskOrStartup("test1", QHash<QString, QVariant>());

    QCOMPARE(rowsAboutToBeInsertedSpy.count(), 1);
    QCOMPARE(rowsInsertedSpy.count(), 1);
    QCOMPARE(rowsAboutToBeRemovedSpy.count(), 0);
    QCOMPARE(rowsRemovedSpy.count(), 0);
    QCOMPARE(dataChangedSpy.count(), 0);
    QCOMPARE(layoutChangedSpy.count(), 0);
    QCOMPARE(modelResetSpy.count(), 0);

    myModel.addTaskOrStartup("test2", QHash<QString, QVariant>());

    QCOMPARE(rowsAboutToBeInsertedSpy.count(), 2);
    QCOMPARE(rowsInsertedSpy.count(), 2);
    QCOMPARE(rowsAboutToBeRemovedSpy.count(), 0);
    QCOMPARE(rowsRemovedSpy.count(), 0);
    QCOMPARE(dataChangedSpy.count(), 0);
    QCOMPARE(layoutChangedSpy.count(), 0);
    QCOMPARE(modelResetSpy.count(), 0);

    myModel.addTaskOrStartup("test3", QHash<QString, QVariant>());

    QCOMPARE(rowsAboutToBeInsertedSpy.count(), 3);
    QCOMPARE(rowsInsertedSpy.count(), 3);
    QCOMPARE(rowsAboutToBeRemovedSpy.count(), 0);
    QCOMPARE(rowsRemovedSpy.count(), 0);
    QCOMPARE(dataChangedSpy.count(), 0);
    QCOMPARE(layoutChangedSpy.count(), 0);
    QCOMPARE(modelResetSpy.count(), 0);
    
    myModel.removeTaskOrStartup("test1");
    
    QCOMPARE(rowsAboutToBeInsertedSpy.count(), 3);
    QCOMPARE(rowsInsertedSpy.count(), 3);
    QCOMPARE(rowsAboutToBeRemovedSpy.count(), 1);
    QCOMPARE(rowsRemovedSpy.count(), 1);
    QCOMPARE(dataChangedSpy.count(), 0);
    QCOMPARE(layoutChangedSpy.count(), 0);
    QCOMPARE(modelResetSpy.count(), 0);

    myModel.addTaskOrStartup("test1", QHash<QString, QVariant>());

    QCOMPARE(rowsAboutToBeInsertedSpy.count(), 4);
    QCOMPARE(rowsInsertedSpy.count(), 4);
    QCOMPARE(rowsAboutToBeRemovedSpy.count(), 1);
    QCOMPARE(rowsRemovedSpy.count(), 1);
    QCOMPARE(dataChangedSpy.count(), 0);
    QCOMPARE(layoutChangedSpy.count(), 0);
    QCOMPARE(modelResetSpy.count(), 0);
}

void TasksModelUnitTests::testTaskRemove()
{
    TasksModel myModel;
    
    myModel.addTaskOrStartup("test1", QHash<QString, QVariant>());
    myModel.addTaskOrStartup("test2", QHash<QString, QVariant>());
    myModel.addTaskOrStartup("test3", QHash<QString, QVariant>());

    QSignalSpy rowsAboutToBeInsertedSpy(&myModel, SIGNAL(rowsAboutToBeInserted(const QModelIndex&, int, int)));
    QSignalSpy rowsInsertedSpy(&myModel, SIGNAL(rowsInserted(const QModelIndex&, int, int)));
    QSignalSpy rowsAboutToBeRemovedSpy(&myModel, SIGNAL(rowsAboutToBeRemoved(const QModelIndex&, int, int)));
    QSignalSpy rowsRemovedSpy(&myModel, SIGNAL(rowsRemoved(const QModelIndex&, int, int)));
    QSignalSpy dataChangedSpy(&myModel, SIGNAL(dataChanged(const QModelIndex&, const QModelIndex&)));
    QSignalSpy layoutChangedSpy(&myModel, SIGNAL(layoutChanged()));
    QSignalSpy modelResetSpy(&myModel, SIGNAL(modelReset()));

    QVERIFY(rowsAboutToBeInsertedSpy.isValid());
    QVERIFY(rowsInsertedSpy.isValid());
    QVERIFY(rowsAboutToBeRemovedSpy.isValid());
    QVERIFY(rowsRemovedSpy.isValid());
    QVERIFY(dataChangedSpy.isValid());
    QVERIFY(layoutChangedSpy.isValid());
    QVERIFY(modelResetSpy.isValid());
    
    myModel.removeTaskOrStartup("test2");
    
    QCOMPARE(rowsAboutToBeInsertedSpy.count(), 0);
    QCOMPARE(rowsInsertedSpy.count(), 0);
    QCOMPARE(rowsAboutToBeRemovedSpy.count(), 1);
    QCOMPARE(rowsRemovedSpy.count(), 1);
    QCOMPARE(dataChangedSpy.count(), 0);
    QCOMPARE(layoutChangedSpy.count(), 0);
    QCOMPARE(modelResetSpy.count(), 0);
    
    myModel.removeTaskOrStartup("test1");
    
    QCOMPARE(rowsAboutToBeInsertedSpy.count(), 0);
    QCOMPARE(rowsInsertedSpy.count(), 0);
    QCOMPARE(rowsAboutToBeRemovedSpy.count(), 2);
    QCOMPARE(rowsRemovedSpy.count(), 2);
    QCOMPARE(dataChangedSpy.count(), 0);
    QCOMPARE(layoutChangedSpy.count(), 0);
    QCOMPARE(modelResetSpy.count(), 0);

    myModel.addTaskOrStartup("test2", QHash<QString, QVariant>());

    QCOMPARE(rowsAboutToBeInsertedSpy.count(), 1);
    QCOMPARE(rowsInsertedSpy.count(), 1);
    QCOMPARE(rowsAboutToBeRemovedSpy.count(), 2);
    QCOMPARE(rowsRemovedSpy.count(), 2);
    QCOMPARE(dataChangedSpy.count(), 0);
    QCOMPARE(layoutChangedSpy.count(), 0);
    QCOMPARE(modelResetSpy.count(), 0);
    
    myModel.removeTaskOrStartup("test3");
    
    QCOMPARE(rowsAboutToBeInsertedSpy.count(), 1);
    QCOMPARE(rowsInsertedSpy.count(), 1);
    QCOMPARE(rowsAboutToBeRemovedSpy.count(), 3);
    QCOMPARE(rowsRemovedSpy.count(), 3);
    QCOMPARE(dataChangedSpy.count(), 0);
    QCOMPARE(layoutChangedSpy.count(), 0);
    QCOMPARE(modelResetSpy.count(), 0);
    
    myModel.removeTaskOrStartup("test2");
    
    QCOMPARE(rowsAboutToBeInsertedSpy.count(), 1);
    QCOMPARE(rowsInsertedSpy.count(), 1);
    QCOMPARE(rowsAboutToBeRemovedSpy.count(), 4);
    QCOMPARE(rowsRemovedSpy.count(), 4);
    QCOMPARE(dataChangedSpy.count(), 0);
    QCOMPARE(layoutChangedSpy.count(), 0);
    QCOMPARE(modelResetSpy.count(), 0);
}

/**
 * \todo Implement it
 */
void TasksModelUnitTests::testTaskUpdate()
{
    TasksModel myModel;

    QSignalSpy rowsAboutToBeInsertedSpy(&myModel, SIGNAL(rowsAboutToBeInserted(const QModelIndex&, int, int)));
    QSignalSpy rowsInsertedSpy(&myModel, SIGNAL(rowsInserted(const QModelIndex&, int, int)));
    QSignalSpy rowsAboutToBeRemovedSpy(&myModel, SIGNAL(rowsAboutToBeRemoved(const QModelIndex&, int, int)));
    QSignalSpy rowsRemovedSpy(&myModel, SIGNAL(rowsRemoved(const QModelIndex&, int, int)));
    QSignalSpy dataChangedSpy(&myModel, SIGNAL(dataChanged(const QModelIndex&, const QModelIndex&)));
    QSignalSpy layoutChangedSpy(&myModel, SIGNAL(layoutChanged()));
    QSignalSpy modelResetSpy(&myModel, SIGNAL(modelReset()));

    QVERIFY(rowsInsertedSpy.isValid());
    QVERIFY(rowsRemovedSpy.isValid());
    QVERIFY(modelResetSpy.isValid());
}

QTEST_MAIN(TasksModelUnitTests)
